package yrpg.slm.pc.plugin.form;

import kd.bos.algo.DataSet;
import kd.bos.bill.AbstractBillPlugIn;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.entity.datamodel.events.PropertyChangedArgs;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.servicehelper.user.UserServiceHelper;
import kd.sdk.plugin.Plugin;

import java.math.BigDecimal;
import java.util.EventObject;

/**
 * 单据界面插件 考核表打分
 * @author 任兴
 */
public class BehaviorScoreBillPlugin extends AbstractBillPlugIn implements Plugin {

    @Override
    public void afterCreateNewData(EventObject e) {
        // TODO 获取用户，得到学术主管及相关信息赋值，构建单据体和对应的子单据体
        //用户id
        long userId = UserServiceHelper.getCurrentUserId();
        //学术主管 根据当前用户赋值
        QFilter filter = new QFilter("id", QCP.equals, userId);
        DataSet userBills = QueryServiceHelper.queryDataSet(this.getClass().toString(), "bos_user",
                "id,number", new QFilter[]{filter}, null);
        //获取人员档案
        DataSet mktUserBills = QueryServiceHelper.queryDataSet(this.getClass().toString(), "rm_mkt_user",
                "id as mktId,number", null, null);

        //获取人员岗位级别对应表
        DataSet pplBills = QueryServiceHelper.queryDataSet(this.getClass().toString(), "yrpg_pc_ppl",
                "yrpg_superiorid,yrpg_userid as userid,yrpg_salesteam as salesteam,yrpg_jobid as jobid", null, null);

        DataSet thisDataSet = userBills.leftJoin(mktUserBills).on("number", "number").select(new String[]{"id"}, new String[]{"mktId"}).finish();

        DataSet personDataSet = thisDataSet.leftJoin(pplBills.copy()).on("mktId","yrpg_superiorid").select(new String[]{"mktId"},new String[]{"userId"}).finish();

        //销售队伍、考核期间、办事处、办事处主任、

        /*
        this.getModel().setValue("yrpg_salesteam", userSaleSteam );
        this.getModel().setValue("yrpg_check_date", "" );
        this.getModel().setValue("yrpg_office", "" );
        this.getModel().setValue("yrpg_userfield", "" );*/


        //TODO 构建考核人员单据体
        /**
         * 预留 获取学术主管下面的人员
         */
        DynamicObjectCollection personCollection = ORM.create().toPlainDynamicObjectCollection(personDataSet.copy());
        if (personCollection.size() > 0) {

            this.getModel().setValue("yrpg_manager", personCollection.stream().findFirst().get().getLong("mktId"));
            //int rows = this.getModel().getEntryRowCount("yrpg_assessor");
            int i = 0;
            for(DynamicObject person : personCollection){
                this.getModel().batchCreateNewEntryRow("yrpg_assessor", 1);
                this.getModel().setEntryCurrentRowIndex("yrpg_assessor",i);

                long personId = person.getLong("userId");
                this.getModel().setValue("yrpg_check_userid", personId, i);

                // TODO 构建子单据体  行为考核
                DynamicObjectCollection behaviorCollection = getBehaviorCol(pplBills.copy(),personId);
                int j = 0;
                for (DynamicObject behaviorCol : behaviorCollection) {
                    long  behaviorId = behaviorCol.getLong("id");
                    this.getModel().batchCreateNewEntryRow("yrpg_assessmentitem", 1);
                    this.getModel().setValue("yrpg_check_dimensionid", behaviorId, j , i);
                    j++;
                }
                i++;
            }
        }


    }

    @Override
    public void propertyChanged(PropertyChangedArgs e) {
        super.propertyChanged(e);
        String name = e.getProperty().getName();
//        ChangeData changeData = e.getChangeSet()[0];
//        int rowIndex = changeData.getRowIndex();
        //如果行为
        if ("yrpg_ultimatescore".equals(name)) {
            calculateAssignment("yrpg_assessmentitem","yrpg_ultimatescore","yrpg_rewardtotal");
        }
        //合规奖考核最终得分
        if ("yrpg_compliance".equals(name)) {
            calculateAssignment("yrpg_complianceentry","yrpg_compliance","yrpg_comptotal");
        }
        //本期科研进度
        if ("yrpg_now_progress".equals(name)) {
            calculateAssignment("yrpg_projectscoreentry","yrpg_now_progress","yrpg_compliancetotal");
        }


    }

    /**
     * 计算汇总并且赋值到父级的对应字段上 刷新
     * @param entryKey
     * @param needField
     * @param fieldKey
     */
    public void calculateAssignment(String entryKey,String needField,String fieldKey) {
        int i = this.getModel().getEntryCurrentRowIndex("yrpg_assessor");
        DynamicObjectCollection assessorEntry = this.getModel().getDataEntity(true).getDynamicObjectCollection("yrpg_assessor");
        for(DynamicObject assessor : assessorEntry) {
            BigDecimal totalScore = BigDecimal.ZERO;
            DynamicObjectCollection complianceEntry = assessor.getDynamicObjectCollection(entryKey);
            for (DynamicObject compliance : complianceEntry) {

                BigDecimal ultimateScore = compliance.getBigDecimal(needField);
                totalScore = totalScore.add(ultimateScore);
            }
            assessor.set(fieldKey,totalScore);
        }
        this.getView().updateView(fieldKey,i);//刷新

    }

    /**
     * 获取行为奖
     */
    public DynamicObjectCollection getBehaviorCol(DataSet pplBills,long personId){
        DynamicObjectCollection pplCollection = ORM.create().toPlainDynamicObjectCollection(pplBills.filter("userid = " + personId));
        long saleSteam = pplCollection.stream().findFirst().get().getLong("salesteam");
        long jobId = pplCollection.stream().findFirst().get().getLong("jobid");
        QFilter salesFilter = new QFilter("yrpg_businessline", QCP.equals, saleSteam);
        QFilter jobFilter = new QFilter("yrpg_post", QCP.equals, jobId);
        DynamicObjectCollection behaviorCollection = QueryServiceHelper.query("yrpg_pc_behaviorassess", "id", new QFilter[]{salesFilter, jobFilter});
        return  behaviorCollection;
    }


}